'============================================================
'  Generates general town and city info for use on continent
'  scale maps.  Each town is assigned a town size from 1 to 3,
'  with 3 being the largest.  When you call maketown(), you'll
'  need to tell it how large a town you want.
'============================================================
#append nbos\RandomWords.vbs
#append nbos\CommonUtils.vbs
#append nbos\Taverns.vbs

'define the location of the tables here
TownInfo_godtable = "nbos\data\Gods.dat"
TownInfo_RulersTable = "nbos\data\Rulers.dat"
TownInfo_IndustryTable = "nbos\data\Industry.dat"
TownInfo_WeaponLawsTable = "nbos\data\TownWeaponLaws.dat"
TownInfo_TownDescriptionTable = "nbos\data\TownDescriptions.dat"
TownInfo_RulerDescriptionTable = "nbos\data\RulerDescription.dat"
TownInfo_AdventureHooks = "nbos\data\AdventureHooks.dat"


'o is the object to assign notes to, 
'SizeCode is the size code from 1 to 3
'sLang is the language to use for generating names (Danish, Finnish, etc)
Sub MakeTown( o, SizeCode, sLang)
   Dim n
   o.Text = RandomTownName( sLang)
   o.Notes = TownNotes( SizeCode, o.Text, sLang)
End Sub



Function TownNotes( SizeCode, TownName, sLang)
	dim RulerName
	dim Notes
	dim RulersGod
	
	Notes = ""
	
	RulerName = GetRulerName( SizeCode, sLang)
	RulersGod = GetRulersGod()
	
	Notes = Notes + "Population: " + CStr( GetTownPopulation( SizeCode)) + GetTownDemographics( SizeCode) + CRLF
	Notes = Notes + "Ruler: " + RulerName + CRLF
	Notes = Notes + "Ruler's God: " + RulersGod + CRLF
	Notes = Notes + "Chief Industry: " + GetTownIndustry( SizeCode) + CRLF
	Notes = Notes + "Churches: " + GetTownChurches( SizeCode, RulersGod) + CRLF
	Notes = Notes + "Schools: " + GetTownSchools( SizeCode, sLang) + CRLF
	Notes = Notes + "Taverns: " + GetTownTaverns( SizeCode) + CRLF
	Notes = Notes + "Fees:" + GetTownFees( SizeCode) + CRLF
	Notes = Notes + "Weapon Laws: " + GetTownWeaponLaws( SizeCode) + CRLF
	Notes = Notes + CRLF
	Notes = Notes + GetTownDescription( SizeCode, TownName) + " " + GetRulerDescription( RulerName) + CRLF
	
	Notes = Notes + CRLF
	Notes = Notes + "Adventure Hooks:" + CRLF
	Notes = Notes + GetAdventureHooks( SizeCode, RulerName, sLang) + CRLF
	

     TownNotes = Notes
End Function

Function GetRulerName( SizeCode, sLang)
	Dim s
	Dim n
	Dim t
	dim oRulerTitles
	
	n = ((SizeCode -1) * 4) + d4()
	oRulerTitles = CreateRPGTable()
	oRulerTitles.LoadFromFile TownInfo_RulersTable
	s = oRulerTitles.RandomPick()
	FreeMapObject oRulerTitles

	s = s + " " + RandomName( sLang)
	
	n = d10()
	t = ""
	Select case n
		case 1, 2, 3
			t = ""
		case 4, 5
			t = " II"
		case 6
			t = " III"
		case 7
			t = " IV"
		case 8
			t = " V"
		case 9
			t = " VI"
		case 10
			t = " VII"
	End Select
	
	s = s + t
	
	GetRulerName = s
End Function

Function GetRulersGod
	Dim oRulersGod
	oRulersGod = CreateRPGTable()
	oRulersGod.LoadFromFile TownInfo_godtable
	GetRulersGod =oRulersGod.RandomPick()
	FreeMapObject oRulersGod
End Function


Function GetTownPopulation( SizeCode)
Dim n
	Select Case SizeCode
		Case 3 ' over 10k
			n = (RollDice( 1, 1, 100) * 100) + 10000
		Case 2 ' 2 to 10k
			n = (RollDice( 1, 1, 80) * 100) + 2000
		Case else  ' under 2k
			n = (RollDice( 1, 1, 200) * 10)
		End Select
		GetTownPopulation = n
End Function


Function GetTownDemographics( SizeCode)
   Dim nElf
   Dim nDwarf
   Dim nHalfling
   Dim nGnome
   Dim isNonhuman
   Dim s

   nElf = (SizeCode * 3) + (d8() - 4)
   if nElf > 0 then s = s + "Elf - " + CStr( nElf) + "%"

   nDwarf = (SizeCode * 3) + (d6() -3)
   if nDwarf > 0 then
           if len( s) > 1 then s = s + ", "
           s = s + "Dwarf - "+  CStr( nDwarf) + "%"
   end if

   nHalfling = (SizeCode * 2) + (d6() -3)
   if nHalfling > 0 then
           if len( s) > 1 then s = s + ", "
           s = s + "Halfling - "+  CStr( nHalfling) + "%"
   end if

   nGnome = (SizeCode * 1) + (d4() -2)
   if nGnome > 0 then
           if len( s) > 1 then s = s + ", "
           s = s + "Gnome - "+  CStr( nGnome) + "%"
   end if

         if len( s) > 1 then s = " (" + s + ")"

   GetTownDemographics = s
End Function



Function GetTownIndustry( SizeCode)
        Dim oIndustry
        
        oIndustry = CreateRPGTable()
        oIndustry.LoadFromFile TownInfo_IndustryTable
        GetTownIndustry = oIndustry.RandomPick()
        FreeMapObject oIndustry
End Function

Function GetTownChurches( SizeCode, RulersGod)
	Dim n
	Dim i
	Dim s
	Dim oGods
	
	oGods = CreateRPGTable()
	oGods.LoadFromFile TownInfo_godtable
	
	
	n = (SizeCode -1) * 4 + RollDice(1,1,3) -2
	
	s = ""
	for i = 1 to n
		if i = 1 then
			s = s + RulersGod
		else
			s = s + oGods.RandomPick()
		end if
	
		if i < n then
			s = s + ","
		end if
	next
	if len( s) < 1 then s = "No churches or chapels."
	GetTownChurches = s
	
	FreeMapObject oGods
	
End Function



Function GetTownSchools( SizeCode, sLang)
	Dim n, m
	Dim i
	Dim s
	
	s = ""
	n = ((SizeCode -1) * 2) + (d6() - 4)
	for i = 1 to n
		m = d8()
		Select Case m
			Case 1
				s = s + Proper(RandomWord( sLang)) + " University"
			Case 2
				s = s + "St. " + Proper( RandomWord( sLang)) +  "'s Seminary"
			Case 3
				s = s + "The " + Proper( RandomWord( sLang)) + " College of " + Proper( RandomWord( sLang))
			Case 4
				s = s + "The "  + Proper( RandomWord( sLang)) + " Academy"
			Case 5
				s = s + "The "  + Proper( RandomWord( sLang)) + " Military Academy"
			Case 6
				s = s + "The "  + Proper( RandomWord( sLang)) + " School"
			Case 7
				s = s + "The "  + Proper( RandomWord( sLang)) + " Library and Academy"
			Case 8
				s = s + "The "  + Proper( RandomWord( sLang)) + " Academy for Women"
		End Select
		if i < n then
			s = s + ","
		end if
	next
	
	if n < 1 then
		s = "Primary schools only.  No institutions of higher learning."
	end if
	GetTownSchools = s
End Function

Function GetTownTaverns( SizeCode)
	Dim n, i
   Dim s

	n = ((SizeCode -1) * 2) + (d6() - 3)
	for i = 1 to n
		s = s + RandomTavernName()
		if i < n then s = s + ", "
	next
	
	if len( s) < 1 then
		s = "No taverns!"
	end if
	
	GetTownTaverns = s
End Function


Function GetTownFees( SizeCode)
	Dim s
	Dim n
	s = ""
	
	n = SizeCode * 20
	if (d100() <= n) then
		s = s + "Entry toll (per person): " + CStr( RollDice( SizeCode, 1, 3)) + "cp"
	end if
	
	if (d100() <= n) then
		if Len( s) > 1 then s = s + ", "
			s = s + "Entry toll (per cart): " + CStr( RollDice( SizeCode, 1, 6)) + "cp"
	end if
	
	if (d100() <= n) then
		if Len( s) > 1 then s = s + ", "
			s = s + "Entry toll (per work animal): " + CStr( RollDice( SizeCode, 1, 3)) + "cp"
	end if
	
	n = SizeCode * 10
	if (d100() <= n) then
		if Len( s) > 1 then s = s + ", "
		s = s + "All transactions within the city must be made with the city's official currency (Money changing fees will exist)"
	end if
	
	if (d100() <= (SizeCode * 30)) then
		if Len( s) > 1 then s = s + ", "
		s = s + "Guild fees officially enforced"
	end if
	
	
	if (d100() <= (SizeCode * 5)) then
		if Len( s) > 1 then s = s + ", "
		s = s + "Guards often insist on a small gratuity"
	end if
	
	if len( s) < 1 then s = s + "No tolls or fees!"
	
	GetTownFees = s
End Function

Function GetTownWeaponLaws( SizeCode)
	Dim n
	Dim s
	Dim oTownWeaponLaws


	n = RollDice( 1, 14, 0) + (3 - SizeCode)
	if n < 1 then n = 1
	
	oTownWeaponLaws = CreateRPGTable()
	oTownWeaponLaws.LoadFromFile TownInfo_WeaponLawsTable
	s = oTownWeaponLaws.GetItem(n)
	GetTownWeaponLaws = s
	FreeMapObject oTownWeaponLaws
End Function


Function GetTownDescription( SizeCode, TownName)
	Dim s
   Dim oTownDescription
        
        
   oTownDescription = CreateRPGTable()
   oTownDescription.LoadFromFile TownInfo_TownDescriptionTable
   s = oTownDescription.RandomPick
   s = Replace( s, "%TN%", TownName)
   FreeMapObject oTownDescription
   
   GetTownDescription = s
End Function


Function GetRulerDescription( RulerName)
   Dim s
   Dim oRulerDescription
   
   oRulerDescription = CreateRPGTable()
   oRulerDescription.LoadFromFile TownInfo_RulerDescriptionTable
   
   s = oRulerDescription.RandomPick
   s = Replace( s, "%R%", RulerName)
   GetRulerDescription = s
  
   FreeMapObject oRulerDescription
End Function




Function GetAdventureHooks( SizeCode, RulersName, sLang)
	Dim i
	Dim s, h
	Dim oAdventureHooks
	
	oAdventureHooks = CreateRPGTable()
	oAdventurehooks.LoadFromFile TownInfo_AdventureHooks
	
	s = ""
	For i = 1 to SizeCode
		h = oAdventureHooks.RandomPick()
		h = Replace( h, "%R%", RulersName)
		h = Replace( h, "%RN%", RandomName( sLang))
		s = s + h + CRLF
	Next
	
	FreeMapObject oAdventureHooks
	
	GetAdventureHooks = s
End Function
